
Copter = newClass("Copter", Collidable)
Copter.SMOKE_COLOR0 = love.graphics.newColor(64, 64, 64, 128)
Copter.SMOKE_COLOR1 = love.graphics.newColor(0, 0, 0, 0)

function Copter:init(stage, player, x, y)
	self.stage = stage
	self.player = player
	self.dead = false
	self.damage = 0.0
	self.damageBuffer = 0.0
	self.hitBuffer = 0
	self.firstHitTime = 0
	self.defense = 15.0
	self.resistance = 1.1
	self.speed = 2.5
	self.hasWind = true
	-- Load
	self.sprite = love.graphics.newAnimation(Resource:getImage("data/copter.png"), 56, 58, 0.05)
	self.sprite:setCenter(0, -4.5)
	self.smoke = love.graphics.newParticleSystem(Resource:getImage("data/smoke.png"), 100)
	self.smoke:setLifetime(-1)
	self.smoke:setEmissionRate(10.0)
	self.smoke:setSpread(45.0)
	self.smoke:setDirection(90)
	self.smoke:setSpeed(10, 20)
	self.smoke:setGravity(-40.0)
	self.smoke:setColor(self.SMOKE_COLOR0, self.SMOKE_COLOR1) 
	-- Create body and shape(s)
	local w = self.stage:getWorld()
	local s = self.stage:getScale()
	self.body = love.physics.newBody(w, x * s, y * s)
	self.body:setDamping(0.8)
	self.shape = love.physics.newRectangleShape(self.body, 0 * s, -3 * s, 20 * s, 32 * s)
	self.shape:setData(self)
	self.body:setMassFromShapes()
	-- Create weapon
	self.weapon = Weapon:new(stage, self)
end

function Copter:update(dt)
	if (not self.dead) then
		-- Take damage
		if (self.hitBuffer > 0 and (self.hitBuffer >= 3 or self.firstHitTime + 0.25 <= love.timer.getTime())) then
			local damage = math.floor(self.damageBuffer / self.hitBuffer + 1)
			self.damageBuffer = 0.0
			self.hitBuffer = 0
			self.damage = self.damage + damage
			self.stage:ShowDamage(damage, self:getX(), self:getY())
		end
		-- Update physics
		self.body:applyImpulse(0, -0.71 * (1.0 - math.max(0, self.damage - 80) / 100))
		self.body:setAngularDamping(math.max(0.05, 1.0 - 0.8 * self.damage / 100))
		self.body:applyTorque(self.body:getAngle() * -1.0)
		self.body:applyImpulse(self.player:getXAxis() * self.speed, self.player:getYAxis() * self.speed)
		if (self.hasWind and self.player:isADown()) then self:useWind() end
		-- Animation
		self.sprite:update(dt)
	end
	-- Smoke
	self.smoke:setParticleLife(self.damage * 1.0 / 100 + 1.0, self.damage * 3.0 / 100 + 1.5)
	self.smoke:setSize(self.damage * 0.5 / 100 + 0.5, self.damage * 4.0 / 100 + 0.8, 0.2)
	self.smoke:setPosition(self:getX(), self:getY() + 16)
	self.smoke:update(dt)
	-- Weapon
	self.weapon:update(dt)
end

function Copter:draw()
	love.graphics.draw(self.smoke, 0, 0)
	love.graphics.setColor(255, 255, 255)
	love.graphics.draw(self.sprite, self:getX(), self:getY(), self:getAngle())
	if DEBUG then
		love.graphics.setColor(0, 0, 255)
		love.graphics.polygon(love.draw_line, self.shape:getPoints())
		love.graphics.circle(love.draw_line, self:getX(), self:getY(), 150, 128)
	end
	self.weapon:draw()
end

function Copter:onCollision(collidable, contact)
	if (not self.dead) then
		local vx, vy = contact:getVelocity()
		local v = math.sqrt(vx * vx + vy * vy)
		if (v > self.defense) then
			local damage = (v - self.defense) / self.resistance
			if (self.hitBuffer == 0) then
				self.firstHitTime = love.timer.getTime()
			end
			self.damageBuffer = self.damageBuffer + damage
			self.hitBuffer = self.hitBuffer + 1
			local s = self.stage:getScale()
			local x, y = contact:getPosition()
			self.stage:Spark(x / s, y / s)
		end
	end
end

function Copter:useWind()
	self.hasWind = false
	self.body:setVelocity(0, 0)
	self.stage:makeWind(self)
end

function Copter:getX()
	return self.body:getX() / self.stage:getScale()
end

function Copter:getY()
	return self.body:getY() / self.stage:getScale()
end

function Copter:getAngle()
	return self.body:getAngle()
end

function Copter:getBody()
	return self.body
end

function Copter:getDamage()
	return self.damage
end

function Copter:die()
	self.damage = 200
	self.dead = true
	self.stage:Explode(self:getX(), self:getY())
end

function Copter:isDead()
	return self.dead
end
