
Weapon = newClass("Weapon", Collidable)

function Weapon:init(stage, owner)
	self.stage = stage
	self.owner = owner
	-- Load
	self.sprite = Resource:getImage("data/weapon.png")
	self.sprite:setCenter(19.5, 6.5)
	-- Create body and shape(s)
	local w = self.stage:getWorld()
	local s = self.stage:getScale()
	self.body = love.physics.newBody(w, owner:getX() * s, owner:getY() * s)
	self.body:setAngularDamping(0.05)
	self.shapes = {}
	local shape
	-- Handle
	shape = love.physics.newRectangleShape(self.body, 0 * s, 32 * s, 7 * s, 70 * s)
	shape:setData(self)
	table.insert(self.shapes, shape)
	-- Tip
	shape = love.physics.newRectangleShape(self.body, 0 * s, 56 * s, 26 * s, 16 * s)
	shape:setData(self)
	table.insert(self.shapes, shape)
	self.body:setMassFromShapes()
	local massX, massY = self.body:getLocalCenter()
	self.body:setMass(massX, massY, self.body:getMass() * 0.1, self.body:getInertia()) 
	-- Attach it
	self.joint = love.physics.newRevoluteJoint(self.body, self.owner:getBody(), self.body:getX(), self.body:getY())
end

function Weapon:update(dt)
	if (self.joint ~= nil and self.owner:isDead()) then
		self.joint:destroy()
		self.joint = nil
	end
end

function Weapon:draw()
	love.graphics.setColor(255, 255, 255)
	love.graphics.draw(self.sprite, self:getX(), self:getY(), self:getAngle())
	if DEBUG then
		love.graphics.setColor(0, 0, 255)
		for i, shape in pairs(self.shapes) do
			love.graphics.polygon(love.draw_line, shape:getPoints())
		end
	end
end

function Weapon:onCollision(collidable, contact)
	
end

function Weapon:getX()
	return self.body:getX() / self.stage:getScale()
end

function Weapon:getY()
	return self.body:getY() / self.stage:getScale()
end

function Weapon:getAngle()
	return self.body:getAngle()
end
